/*
Code for custom midi controller based on teensy LC, built for use in a project called "orchestra of speech".
Features: xy touchpad, 8 sliders, 2 joysticks, 6 pots,12 push buttons with built-in leds.
These are connected like this:
xy touch pad connected directly to teensy;
2 pots connected directly to teensy and 16 pots (sliders, joysticks, pots) connected through two 4051 multiplexers;
4 moomentary push buttons connected directly to teensy and 8 buttons throu one 4051 multiplexer;
4 leds connected directly to teensy, and 8 leds connected through one 595 shift register
Update for v2 of controller: added 2 x 8 preset pushbuttons connected to two 4051’s, and two init pushbuttons connected directly to teensy.

Daniel Formo 2017 daniel.formo@ntnu.no
http://orchestraofspeech.com
*/

#include "TouchScreen.h"

//threshold minimum analog change
int THRESH = 4;

byte MIDIChannel = 1;

int debug = 0; //enable serial monitoring

//touchscreen pins
const int xm = A0;//must be analog pin
const int yp = A1;//must be analog pin
const int xp = A2;//can be digital
const int ym = A3; //can be digital

//touchscreen x resistance = 551 ohms
TouchScreen ts = TouchScreen(xm, yp, xp, ym, 551);

//midi control change numbers
byte cc[20]={0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19};

//memory holders for controller values
int16_t controller[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int16_t filtered[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int16_t scaled[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int16_t previous[20]={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

//memory for values in filter
int16_t delta[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
int16_t current[20] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};


//directly connected button pins
byte buttonPin[6]={6,8,10,12,24,26};
byte buttonState[6]={HIGH, HIGH, HIGH, HIGH, HIGH, HIGH};
byte lastButtonState[6]={HIGH, HIGH, HIGH, HIGH, HIGH, HIGH};


//directly connected led pins
byte ledPin[4]={7,9,11,13};


//about debouncing: pushbuttons are not used for time critical operations so buttons are updated slowly enough to bypass bouncing altogether

//button multiplexers common const
const byte selPin[] = {0, 1, 2}; //sel pins
const byte btnCount = 8; //Number of buttons
int muxChannel;
byte noteState[12]={LOW,LOW,LOW,LOW,LOW,LOW,LOW,LOW,LOW,LOW,LOW,LOW};

//button multiplexers individual constants
const byte muxPin = A6;  //y pin
const byte muxPin2 = A9; //y-pin of second pushbutton array
const byte muxPin3 = A11; //y-pin of third pushbutton array
int btnReading;
int btnReading2;
int btnReading3;
int btnState[btnCount];
int btnState2[btnCount];
int btnState3[btnCount];

//led shiftregister pins
int dataPin = 5;
int latchPin = 4;
int clockPin = 3;

byte leds = 0;

// initialize values of selct pins
int r0 = 0;     
int r1 = 0;     
int r2 = 0;


//===============================

//FUNCTIONS


//read buttons from multiplexer

byte readMux(byte muxChannel)
{
  // set the selector pins HIGH and LOW to match the binary value of channel
  for(int bit = 0; bit < 3; bit++){
    int pin = selPin[bit]; // the pin wired to the multiplexer select bit
    int isBitSet =  bitRead(muxChannel, bit); // true if given bit set in channel
    digitalWrite(pin, isBitSet);
    //sets high/low threshold
    delay(0.1); //allow voltages to settle before reading
    if (digitalRead(muxPin) == HIGH) btnReading = LOW;
    else btnReading = HIGH;
  }
  return btnReading;
} 

//read buttons from multiplexer 2

byte readMux2(byte muxChannel)
{
  // set the selector pins HIGH and LOW to match the binary value of channel
  for(int bit = 0; bit < 3; bit++){
    int pin = selPin[bit]; // the pin wired to the multiplexer select bit
    int isBitSet =  bitRead(muxChannel, bit); // true if given bit set in channel
    digitalWrite(pin, isBitSet);
    delay(0.1); //allow voltages to settle before reading
    if (digitalRead(muxPin2) == HIGH) btnReading2 = LOW;
    else btnReading2 = HIGH;
  }
  return btnReading2;
} 

//read buttons from multiplexer 3
byte readMux3(byte muxChannel)
{
  // set the selector pins HIGH and LOW to match the binary value of channel
  for(int bit = 0; bit < 3; bit++){
    int pin = selPin[bit]; // the pin wired to the multiplexer select bit
    int isBitSet =  bitRead(muxChannel, bit); // true if given bit set in channel
    digitalWrite(pin, isBitSet);
    delay(0.1); //allow voltages to settle before reading
    if (digitalRead(muxPin3) == HIGH) btnReading3 = LOW;
    else btnReading3 = HIGH;
  }
  return btnReading3;
} 



//update shift register

void updateShiftRegister()
{
   digitalWrite(latchPin, LOW);
   shiftOut(dataPin, clockPin, LSBFIRST, leds);
   digitalWrite(latchPin, HIGH);
}


//usbMIDI receive callback functions: onNoteON, onNoteOff

void OnNoteOn(byte channel, byte note, byte velocity) {
  if((note >= 60) && (note < 72)) {   //if incoming note is within range 
    if(noteState[note - 60] == LOW) { //and if corresponding note state is OFF
      noteState[note - 60] = HIGH;    //set note state to ON
      if (note < 68) {                //if corresponding led is connected to shift reg
        bitSet(leds, -note + 67);    //set (and reverse) bit in ledstate byte
        updateShiftRegister();  //and update shift register to switch on led
      }else{                 
        digitalWrite(ledPin[note - 68], HIGH);  //else update the directly connected led
      }
    }
      if (debug == 1) {
      Serial.print("usbmidi noteon ");
      Serial.println(note, DEC);
    }
  }
}

void OnNoteOff(byte channel, byte note, byte velocity) {  
 if((note >= 60) && (note < 72)) { 
   if(noteState[note - 60] == HIGH); {
      noteState[note - 60] = LOW;
      if (note < 68) {
        bitClear(leds, -note + 67);      
        updateShiftRegister();
      }else{
        digitalWrite(ledPin[note - 68], LOW);
      }
    }
    if (debug == 1) {
      Serial.print("usbmidi noteoff ");
      Serial.println(note, DEC);
    }
  }
}

//filter function to only send values if it changes more than the set threshold
int16_t filter( int16_t x, int n )
{
  delta[n] = x - current[n];

  if ( delta[n] <= -THRESH )
  current[n] = x + THRESH;

  if ( delta[n] >= THRESH )
  current[n] = x - THRESH;

  return current[n];
}


//===========================================
//SETUP
 
void setup() {

 if (debug == 1) {
  Serial.begin(9600);
 }

//count through and set pin modes for directly connected buttons
for (int j=0; j<=5; j++) {
  pinMode(buttonPin[j], INPUT_PULLUP);
  }

//count trhough and set pin modes for directly connected leds
for (int j=0; j<=3; j++) {
  pinMode(ledPin[j], OUTPUT);
  }

  
//set shift register pin modes
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);  
  pinMode(clockPin, OUTPUT);
  updateShiftRegister();
  leds = 0; //set all leds to off after power on

//set pins modes for y-pin on button multiplexers
    pinMode(A6, INPUT_PULLUP);
    pinMode(A9, INPUT_PULLUP);
    pinMode(A11, INPUT_PULLUP);
//set pin modes for multiplexer sel pins
  for (byte thisPin = 0; thisPin < 3; thisPin++)  { //count through array and assign and name the pins
    pinMode(selPin[thisPin], OUTPUT);
  } 

//initialize current value in pot filter
for(int k=0;k<20;k++){
  current[k] = THRESH;
}


//set usbMIDI handles to receive noteon and noteoff
  usbMIDI.setHandleNoteOff(OnNoteOff);
  usbMIDI.setHandleNoteOn(OnNoteOn);

}

//loop timer
elapsedMillis msec = 0;

//===================================

void loop() {

  
  if (msec >= 10) {         //update every 10 ms

    TSPoint p = ts.getPoint();
      if (p.z > ts.pressureThreshhold) {
        if (debug == 1) {
          Serial.print("x "); Serial.print(p.x);
          Serial.print(" y "); Serial.print(p.y);
          Serial.print(" p "); Serial.println(p.z);
        }
        
        //map to midi range
        p.x = constrain(map(p.x, 210, 820, 0, 127), 0, 127);
        p.y = constrain(map(p.y, 140, 910, 0, 127), 0, 127);

        controller[18] = p.y;
        controller[19] = p.x;
        
        //send midi cc  from xy pad if values change
        if (controller[18] != previous[18]) {
          usbMIDI.sendControlChange(cc[18], controller[18], MIDIChannel);     
          previous[18] = controller[18];        
        }
        if (controller[19] != previous[19]) {
          usbMIDI.sendControlChange(cc[19], controller[19], MIDIChannel);
          previous[19] = controller[19];    
        }
      }

    //read buttons 1-8 connected to demultiplexer
    for (byte i = 0; i < btnCount; i++){     
           
      btnState[i] = readMux(i);      //read state of each button
      
      if(btnState[i] != noteState[i]) {    //if state is not same as stored noteState
        if (noteState[i] == LOW) {          //and if the corresponding note state is off
        usbMIDI.sendNoteOn(60 + i, 127, MIDIChannel); //send noteon (start from note 60)
        noteState[i] = HIGH;                          //set note state to high
        bitSet(leds, -i +7);    //set bit in ledstate byte (bits flipped due to wiring)
        updateShiftRegister();  //update shift register to switch on led
        if (debug == 1) {
          Serial.print("note ON  ");
          Serial.println(60 + i);
          }
        }else{                    //or if note state was already on
        usbMIDI.sendNoteOff(60 + i, 0, MIDIChannel);  //send noteoff
        noteState[i] = LOW;
        bitClear(leds, -i +7);
        updateShiftRegister();
        if (debug == 1) {
          Serial.print("note OFF ");         
          Serial.println(60 + i);
          }
        }
 
      } 
     }


//read buttons 1-8 connected to demultiplexer2
    for (byte i = 0; i < btnCount; i++){
                      
      if(readMux2(i) != btnState2[i]) {   
        if(btnState2[i] == HIGH) {
          usbMIDI.sendNoteOn(72 + i, 127, MIDIChannel); //send noteon (start from note 72)
          if (debug == 1) {
            Serial.print("note ON  "); Serial.println(72 + i);
            }
        }else{
          usbMIDI.sendNoteOff(72 + i, 0, MIDIChannel);  //send noteoff
          if (debug == 1) {
            Serial.print("note OFF "); Serial.println(72 + i);
            }
         }
       }
     } 



//read buttons for demultiplexer 3
    for (byte i = 0; i < btnCount; i++){

      if(readMux3(i) != btnState3[i]) {   
        if(btnState3[i] == HIGH) {
          usbMIDI.sendNoteOn(80 + i, 127, MIDIChannel); //send noteon
          if (debug == 1) {
            Serial.print("note ON  ");Serial.println(80 + i);
          }else{
          usbMIDI.sendNoteOff(80 + i, 0, MIDIChannel);  //send noteoff
          if (debug == 1) {
            Serial.print("note OFF ");Serial.println(80 + i);
            }
          }

         } 
       }
    }
     
    //read directly connected toggle buttons 9-12

for (byte i = 0; i < 4; i++){
    buttonState[i] = digitalRead(buttonPin[i]);
    if (buttonState[i] != lastButtonState[i]) {
      if (buttonState[i] == LOW) {
        if(noteState[i+8] == LOW) {
          usbMIDI.sendNoteOn(68 + i, 127, MIDIChannel);
          noteState[i+8] = HIGH;
          if (debug == 1){
            Serial.print("note ON  ");Serial.println(68 + i);
          }          
        }else{
          usbMIDI.sendNoteOff(68 + i, 0, MIDIChannel);
          noteState[i+8] = LOW;
          if (debug == 1){
            Serial.print("note OFF   ");Serial.println(68 + i);
          }
        }
      }
    }
    lastButtonState[i] = buttonState[i];
}

//read last to momentary reset buttons
for (byte i = 0; 0 < 2; i++){
  buttonState[i+4] = digitalRead(buttonPin[i+4]);
  if (buttonState[i+4] != lastButtonState[i+4]) {
    if (buttonState[i+4] == LOW){
      usbMIDI.sendNoteOn(88 + i, 127, MIDIChannel);
      if (debug == 1) {
        Serial.print("note ON  ");Serial.println(88+i);
      }else{
        usbMIDI.sendNoteOff(88 + i, 0, MIDIChannel);
        if (debug == 1){
          Serial.print("note OFF  ");Serial.println(88+i);
        }
      }
    }
  }
  lastButtonState[i+4] = buttonState[i+4];
}



//read analog inputs from demultiplexers
  for (int count=0; count<=7; count++) {
      
      int count8 = count + 8; //setup second demultiplexer index

      // select the bits  
    
      r0 = bitRead(count,0);        
      r1 = bitRead(count,1);       
      r2 = bitRead(count,2);     

      // write bits to multiplexer select pins

     digitalWrite(selPin[0], r0);
     digitalWrite(selPin[1], r1);
     digitalWrite(selPin[2], r2);

 
      //read y pins at current address (count)

      controller[count] = analogRead(A4);
      controller[count8] = analogRead(A5);

      //filter out jitter and bitshift from 10-bit values to 7-bit midi range; 
      scaled[count] = filter(controller[count],count) >> 3;
      scaled[count8] = filter(controller[count8],count8) >> 3;

      // only transmit MIDI messages if analog input changed (more than filter threshold)
      if (scaled[count] != previous[count]) 
       {
        usbMIDI.sendControlChange(cc[count], scaled[count], MIDIChannel);

        if (debug == 1) {
          Serial.print(count);
          Serial.print(" = ");
          Serial.println(controller[count]);
          }
       
        previous[count] = scaled[count];
      }
    
      //same for second multiplexer
         if (scaled[count8] != previous[count8]) 
       {
        if (debug == 1) {
          Serial.print(count8);
          Serial.print(" = ");
          Serial.println(controller[count8]);
          }
        usbMIDI.sendControlChange(cc[count8], scaled[count8], MIDIChannel);
        previous[count8] = scaled[count8];
      }


  }

  //read last two analog inputs
    controller[16] = analogRead(A8);
    scaled[16] = filter(controller[16],16) >> 3;
    
     if (scaled[16] != previous[16]) 
       {
        if (debug == 1) {
          Serial.print("16");
          Serial.print(" = ");
          Serial.println(controller[16]);
        }
        usbMIDI.sendControlChange(cc[16], scaled[16], MIDIChannel);
        previous[16] = scaled[16];
      }

    controller[17] = analogRead(A7);
    scaled[17] = filter(controller[17],17) >> 3;
    
     if (scaled[17] != previous[17]) 
       {
        if (debug == 1) {
          Serial.print("17");
          Serial.print(" = ");
          Serial.println(controller[17]);
        }
        usbMIDI.sendControlChange(cc[17], scaled[17], MIDIChannel);
        previous[17] = scaled[17];
      }
  //reset timer for next cycle of loop    
  msec = 0;
  }

usbMIDI.read(); //read midi coming back from computer

}

